/*
 * Copyright (c) 2005 (Mike) Maurice Kienenberger (mkienenb@alaska.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */


package org.gamenet.application.mm8leveleditor.handler;

import java.awt.BorderLayout;
import java.awt.Component;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import org.gamenet.application.mm8leveleditor.control.EventControl;
import org.gamenet.application.mm8leveleditor.data.mm6.Event;
import org.gamenet.application.mm8leveleditor.lod.LodResource;
import org.gamenet.application.mm8leveleditor.lod.TemplateLodResource;
import org.gamenet.swing.controls.ComponentArrayPanel;
import org.gamenet.util.TaskObserver;
import org.gamenet.util.UnsupportedFileFormatException;

public class EvtHandler extends AbstractBaseHandler
{
    private LodResource sourceLodResource = null;
    private List eventList = null;
    
    public Component getComponentFor(LodResource lodResource, TaskObserver taskObserver) throws InterruptedException
    {
        this.sourceLodResource = lodResource;
        
        taskObserver.taskProgress(lodResource.getName(), 1f / (float)taskObserver.getRange());

        JPanel panel = new JPanel(new BorderLayout());
        JTextArea description = new JTextArea(lodResource.getTextDescription());
        panel.add(description, BorderLayout.PAGE_START);
 
        taskObserver.taskProgress(lodResource.getName(), 2f / (float)taskObserver.getRange());

        byte byteArray[] = null;
        try
        {
            byteArray = lodResource.getData();
        }
        catch (IOException anIOException)
        {
            Throwable throwable = new Throwable("Unable to extract data.", anIOException);
            throwable.printStackTrace();
            StringWriter stringWriter = new StringWriter();
            PrintWriter printWriter = new PrintWriter(stringWriter, true);
            throwable.printStackTrace(printWriter);
            
            Component component = new JTextArea(stringWriter.toString());

            panel.add(component, BorderLayout.CENTER);
            return panel;
        }
        
        taskObserver.taskProgress(lodResource.getName(), 0.7f);
        if (Thread.currentThread().isInterrupted())
            throw new InterruptedException("EvtHandler.getComponentFor() was interrupted.");

        if (false == Event.checkDataIntegrity(byteArray, 0, byteArray.length))
        {
            throw new UnsupportedFileFormatException("Not an expected mm6 evt format.");
        }
	    eventList = Event.readEvents(byteArray);
        
        taskObserver.taskProgress(lodResource.getName(), 0.75f);
        if (Thread.currentThread().isInterrupted())
            throw new InterruptedException("EvtHandler.getComponentFor() was interrupted.");

        JPanel eventHandlerPanel = new JPanel(new BorderLayout());
        
		Component component = eventHandlerPanel;
		
        // eventHandlerPanel.add(new ComparativeTableControl(Event.getOffsetList(), eventList));

		// IMPLEMENT: update # of events value when events are added or deleted
		eventHandlerPanel.add(new JLabel("# of Events: " + String.valueOf(eventList.size())), BorderLayout.PAGE_START);

        List eventControlList = new ArrayList(eventList.size());
        for (Iterator eventIterator = eventList.iterator(); eventIterator.hasNext();)
        {
            Event event = (Event) eventIterator.next();
            eventControlList.add(new EventControl(event));
        }
        
        ComponentArrayPanel eventComponentArrayPanel = new ComponentArrayPanel(eventControlList, new ComponentArrayPanel.ComponentDataSource()
        {
            public Component createComponent(int componentIndex)
            {
                Event newEvent = createNewEvent();
                eventList.add(componentIndex, newEvent);
                return new EventControl(newEvent);
            }

            public void fireComponentAdded(int componentIndex, Component component) {}
            public void fireComponentDeleted(int componentIndex, Component component) {}
            public void fireComponentMovedUp(int componentIndex, Component component) {}
            public void fireComponentMovedDown(int componentIndex, Component component) {}
        });

        // JScrollPane eventScroller = new JScrollPane(eventComponentArrayPanel);
        eventHandlerPanel.add(eventComponentArrayPanel, BorderLayout.CENTER);

        panel.add(component, BorderLayout.CENTER);
        
        taskObserver.taskProgress(lodResource.getName(), 1f);

        return panel;
    }		

    private Event createNewEvent()
    {
        Integer commandIntegerArray[] = Event.getCommands();
        String commandNameArray[] = new String[commandIntegerArray.length];
        for (int commandIndex = 0; commandIndex < commandIntegerArray.length; commandIndex++)
        {
            commandNameArray[commandIndex] = Event.getEventCommandName(commandIntegerArray[commandIndex].intValue());
        }
        
        String s = (String)JOptionPane.showInputDialog(
                            null,
                            "Choose an event command type:\n",
                            "Event Command Wizard",
                            JOptionPane.PLAIN_MESSAGE,
                            null,
                            commandNameArray,
                            commandNameArray[0]);

        if ((s == null) || (s.length() == 0))
        {
            // no command type chosen
            return null;
        }

        int commandValue = -1;
        for (int commandIndex = 0; commandIndex < commandNameArray.length; commandIndex++)
        {
            String value = commandNameArray[commandIndex];
            if (s.equals(value))
            {
                commandValue = commandIntegerArray[commandIndex].intValue();
                break;
            }
        }

        if (-1 == commandValue)
        {
            // no command found
            return null;
        }

        int eventNumber = 0;
        int eventSequence = 0;
        
        Event newEvent = new Event();
        newEvent.initializeWithValues(eventNumber, eventSequence, commandValue);
        
        return newEvent;
    }

    public LodResource getUpdatedLodResource()
    {
        return new EvtLodResource(sourceLodResource, eventList);
    }
    
    public class EvtLodResource extends TemplateLodResource
    {
        private List eventList = null;
        
        public EvtLodResource(LodResource templateLodResource, List eventList)
        {
            super(templateLodResource);
            this.eventList = eventList;
        }
        
        public byte[] getData() throws IOException
        {
            return Event.writeEvents(eventList);
        }
    }
}
