/*
 * Copyright (c) 2005 (Mike) Maurice Kienenberger (mkienenb@alaska.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */


package org.gamenet.application.mm8leveleditor.handler;
import java.awt.BorderLayout;
import java.awt.Component;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextArea;

import org.gamenet.application.mm8leveleditor.lod.LodResource;
import org.gamenet.application.mm8leveleditor.lod.RawDataTemplateLodResource;
import org.gamenet.swing.controls.ResizingJTable;
import org.gamenet.swing.controls.VariableTabularStringTableModel;
import org.gamenet.util.TaskObserver;

public class StrHandler implements LodResourceHandler
{
    private LodResource sourceLodResource = null;
    private VariableTabularStringTableModel stringTableModel = null;
    
    public Component getComponentFor(LodResource lodResource, TaskObserver taskObserver)
    {
        this.sourceLodResource = lodResource;
        
		Component component = null;

        JPanel panel = new JPanel(new BorderLayout());
        JTextArea description = new JTextArea(lodResource.getTextDescription());
        panel.add(description, BorderLayout.PAGE_START);

        byte byteData[] = null;
        try
        {
            byteData = lodResource.getData();
        }
        catch (IOException anIOException)
        {
            Throwable throwable = new Throwable("Unable to extract data.", anIOException);
            throwable.printStackTrace();
            StringWriter stringWriter = new StringWriter();
            PrintWriter printWriter = new PrintWriter(stringWriter, true);
            throwable.printStackTrace(printWriter);
            
            component = new JTextArea(stringWriter.toString());

            panel.add(component, BorderLayout.CENTER);
            return panel;
        }

		String strings[] = convertStrDataToStringArray(byteData);

		this.stringTableModel = new VariableTabularStringTableModel(strings);
		
		JTable dataTable = new ResizingJTable(stringTableModel);
		component = dataTable;


		panel.add(component, BorderLayout.CENTER);
		return panel;
    }

    private String[] convertStrDataToStringArray(byte[] byteData)
    {
        List stringsList = new ArrayList();
        String textFileContents = new String(byteData);
		int textFileIndex = 0;
		StringBuffer textBuffer = new StringBuffer();
		while (textFileIndex < textFileContents.length())
		{
			char textChar = textFileContents.charAt(textFileIndex++);
			
			if (0 == textChar)
			{
			    stringsList.add(textBuffer.toString());
				textBuffer = new StringBuffer();
			}
			else
			{
				textBuffer.append(textChar);
			}
		}
		
        String strings[] = new String[stringsList.size()];
        int row = 0;
        Iterator stringIterator = stringsList.iterator();
        while (stringIterator.hasNext())
        {
            String value = (String) stringIterator.next();
            strings[row++] = value;
        }
        
        return strings;
    }
    
    private byte[] convertStringArrayToStrData(String[] strings)
    {
        StringBuffer textBuffer = new StringBuffer();
        for (int row = 0; row < strings.length; ++row)
        {
            textBuffer.append(strings[row]);
            textBuffer.append((char)0);
        }

		return textBuffer.toString().getBytes();
    }

    public LodResource getUpdatedLodResource()
    {
        String strings[] = this.stringTableModel.getDataAsStringArray();
        return new RawDataTemplateLodResource(sourceLodResource, convertStringArrayToStrData(strings));
    }
}
